@@ -43,9 +43,26 @@ import re
 import sys
 import textwrap
 import typing as t
-from distutils.util import strtobool as strtoint
 
-import pkg_resources
+try:
+    # `importlib_resources` is required for Python versions below 3.12
+    # See more in the package docs: https://pypi.org/project/importlib-resources/
+    try:
+        from importlib_resources import files as import_files
+    except ImportError:
+        from importlib.resources import files as import_files
+
+    def get_resource_bytes(package: str, resource: str) -> bytes:
+        ref = import_files(package).joinpath(resource)
+        return ref.read_bytes()
+
+except ImportError:
+    # The `pkg_resources` is deprecated in Python 3.12
+    import pkg_resources
+
+    def get_resource_bytes(package: str, resource: str) -> bytes:
+        return pkg_resources.resource_string(package, resource)
+
 
 from pypiserver.backend import (
     SimpleFileBackend,
@@ -63,10 +80,29 @@ except ImportError:
     HtpasswdFile = None
 
 
-# The "strtobool" function in distutils does a nice job at parsing strings,
-# but returns an integer. This just wraps it in a boolean call so that we
-# get a bool.
-strtobool: t.Callable[[str], bool] = lambda val: bool(strtoint(val))
+def legacy_strtoint(val: str) -> int:
+    """Convert a string representation of truth to true (1) or false (0).
+
+    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
+    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
+    'val' is anything else.
+
+    The "strtobool" function in distutils does a nice job at parsing strings,
+    but returns an integer. This just wraps it in a boolean call so that we
+    get a bool.
+
+    Borrowed from deprecated distutils.
+    """
+    val = val.lower()
+    if val in ("y", "yes", "t", "true", "on", "1"):
+        return 1
+    elif val in ("n", "no", "f", "false", "off", "0"):
+        return 0
+    else:
+        raise ValueError("invalid truth value {!r}".format(val))
+
+
+strtobool: t.Callable[[str], bool] = lambda val: bool(legacy_strtoint(val))
 
 
 # Specify defaults here so that we can use them in tests &c. and not need
@@ -88,6 +124,9 @@ class DEFAULTS:
     PORT = 8080
     SERVER_METHOD = "auto"
     BACKEND = "auto"
+    SERVER_BASE_URL = (
+        "/"  # if server need to served under example.com/<SERVER_BASE_URL>
+    )
 
 
 def auth_arg(arg: str) -> t.List[str]:
@@ -151,9 +190,7 @@ def health_endpoint_arg(arg: str) -> str:
 def html_file_arg(arg: t.Optional[str]) -> str:
     """Parse the provided HTML file and return its contents."""
     if arg is None or arg == "pypiserver/welcome.html":
-        return pkg_resources.resource_string(__name__, "welcome.html").decode(
-            "utf-8"
-        )
+        return get_resource_bytes(__name__, "welcome.html").decode("utf-8")
     with open(arg, "r", encoding="utf-8") as f:
         msg = f.read()
     return msg
@@ -484,6 +521,14 @@ def get_parser() -> argparse.ArgumentParser:
         ),
     )
 
+    run_parser.add_argument(
+        "--server-base-url",
+        default=DEFAULTS.SERVER_BASE_URL,
+        help=(
+            "Serve all routes under SERVER_BASE_URL prefix (default: {DEFAULTS.SERVER_BASE_URL})"
+        ),
+    )
+
     update_parser = subparsers.add_parser(
         "update",
         help=textwrap.dedent(
@@ -689,6 +734,7 @@ class RunConfig(_ConfigCommon):
         log_req_frmt: str,
         log_res_frmt: str,
         log_err_frmt: str,
+        server_base_url: str,
         auther: t.Optional[t.Callable[[str, str], bool]] = None,
         **kwargs: t.Any,
     ) -> None:
@@ -708,6 +754,7 @@ class RunConfig(_ConfigCommon):
         self.log_req_frmt = log_req_frmt
         self.log_res_frmt = log_res_frmt
         self.log_err_frmt = log_err_frmt
+        self.server_base_url = server_base_url
         # Derived properties
         self._derived_properties = self._derived_properties + ("auther",)
         self.auther = self.get_auther(auther)
@@ -733,6 +780,7 @@ class RunConfig(_ConfigCommon):
             "log_req_frmt": namespace.log_req_frmt,
             "log_res_frmt": namespace.log_res_frmt,
             "log_err_frmt": namespace.log_err_frmt,
+            "server_base_url": namespace.server_base_url,
         }
 
     def get_auther(
